@@ -19,18 +19,19 @@
 
 use std::sync::Arc;
 
+use arrow::array::Array;
 use arrow::datatypes::{DataType as ArrowType, Schema, SchemaRef};
 use arrow::error::Result as ArrowResult;
 use arrow::record_batch::{RecordBatch, RecordBatchReader};
 use arrow::{array::StructArray, error::ArrowError};
 
-use crate::arrow::array_reader::{build_array_reader, ArrayReader, StructArrayReader};
+use crate::arrow::array_reader::{build_array_reader, ArrayReader};
 use crate::arrow::schema::parquet_to_arrow_schema;
 use crate::arrow::schema::{
     parquet_to_arrow_schema_by_columns, parquet_to_arrow_schema_by_root_columns,
 };
-use crate::errors::{ParquetError, Result};
-use crate::file::metadata::ParquetMetaData;
+use crate::errors::Result;
+use crate::file::metadata::{KeyValue, ParquetMetaData};
 use crate::file::reader::FileReader;
 
 /// Arrow reader api.
@@ -78,8 +79,35 @@ pub trait ArrowReader {
         T: IntoIterator<Item = usize>;
 }
 
+#[derive(Debug, Clone, Default)]
+pub struct ArrowReaderOptions {
+    skip_arrow_metadata: bool,
+}
+
+impl ArrowReaderOptions {
+    /// Create a new [`ArrowReaderOptions`] with the default settings
+    fn new() -> Self {
+        Self::default()
+    }
+
+    /// Parquet files generated by some writers may contain embedded arrow
+    /// schema and metadata. This may not be correct or compatible with your system.
+    ///
+    /// For example:[ARROW-16184](https://issues.apache.org/jira/browse/ARROW-16184)
+    ///
+
+    /// Set `skip_arrow_metadata` to true, to skip decoding this
+    pub fn with_skip_arrow_metadata(self, skip_arrow_metadata: bool) -> Self {
+        Self {
+            skip_arrow_metadata,
+        }
+    }
+}
+
 pub struct ParquetFileArrowReader {
     file_reader: Arc<dyn FileReader>,
+
+    options: ArrowReaderOptions,
 }
 
 impl ArrowReader for ParquetFileArrowReader {
@@ -87,10 +115,7 @@ impl ArrowReader for ParquetFileArrowReader {
 
     fn get_schema(&mut self) -> Result<Schema> {
         let file_metadata = self.file_reader.metadata().file_metadata();
-        parquet_to_arrow_schema(
-            file_metadata.schema_descr(),
-            file_metadata.key_value_metadata(),
-        )
+        parquet_to_arrow_schema(file_metadata.schema_descr(), self.get_kv_metadata())
     }
 
     fn get_schema_by_columns<T>(
@@ -106,13 +131,13 @@ impl ArrowReader for ParquetFileArrowReader {
             parquet_to_arrow_schema_by_columns(
                 file_metadata.schema_descr(),
                 column_indices,
-                file_metadata.key_value_metadata(),
+                self.get_kv_metadata(),
             )
         } else {
             parquet_to_arrow_schema_by_root_columns(
                 file_metadata.schema_descr(),
                 column_indices,
-                file_metadata.key_value_metadata(),
+                self.get_kv_metadata(),
             )
         }
     }
@@ -154,14 +179,41 @@ impl ArrowReader for ParquetFileArrowReader {
 }
 
 impl ParquetFileArrowReader {
+    /// Create a new [`ParquetFileArrowReader`]
     pub fn new(file_reader: Arc<dyn FileReader>) -> Self {
-        Self { file_reader }
+        Self {
+            file_reader,
+            options: Default::default(),
+        }
     }
 
-    // Expose the reader metadata
+    /// Create a new [`ParquetFileArrowReader`] with the provided [`ArrowReaderOptions`]
+    pub fn new_with_options(
+        file_reader: Arc<dyn FileReader>,
+        options: ArrowReaderOptions,
+    ) -> Self {
+        Self {
+            file_reader,
+            options,
+        }
+    }
+
+    /// Expose the reader metadata
     pub fn get_metadata(&mut self) -> ParquetMetaData {
         self.file_reader.metadata().clone()
     }
+
+    /// Returns the key value metadata, returns `None` if [`ArrowReaderOptions::skip_arrow_metadata`]
+    fn get_kv_metadata(&self) -> Option<&Vec<KeyValue>> {
+        if self.options.skip_arrow_metadata {
+            return None;
+        }
+
+        self.file_reader
+            .metadata()
+            .file_metadata()
+            .key_value_metadata()
+    }
 }
 
 pub struct ParquetRecordBatchReader {
@@ -183,20 +235,10 @@ impl Iterator for ParquetRecordBatchReader {
                             "Struct array reader should return struct array".to_string(),
                         )
                     });
+
                 match struct_array {
                     Err(err) => Some(Err(err)),
-                    Ok(e) => {
-                        match RecordBatch::try_new(self.schema.clone(), e.columns_ref()) {
-                            Err(err) => Some(Err(err)),
-                            Ok(record_batch) => {
-                                if record_batch.num_rows() > 0 {
-                                    Some(Ok(record_batch))
-                                } else {
-                                    None
-                                }
-                            }
-                        }
-                    }
+                    Ok(e) => (e.len() > 0).then(|| Ok(RecordBatch::from(e))),
                 }
             }
         }
@@ -214,12 +256,6 @@ impl ParquetRecordBatchReader {
         batch_size: usize,
         array_reader: Box<dyn ArrayReader>,
     ) -> Result<Self> {
-        // Check that array reader is struct array reader
-        array_reader
-            .as_any()
-            .downcast_ref::<StructArrayReader>()
-            .ok_or_else(|| general_err!("The input must be struct array reader!"))?;
-
         let schema = match array_reader.get_data_type() {
             ArrowType::Struct(ref fields) => Schema::new(fields.clone()),
             _ => unreachable!("Struct array reader's data type is not struct!"),
@@ -245,18 +281,22 @@ mod tests {
     use rand::{thread_rng, RngCore};
     use serde_json::json;
     use serde_json::Value::{Array as JArray, Null as JNull, Object as JObject};
+    use tempfile::tempfile;
 
     use arrow::array::*;
-    use arrow::datatypes::{DataType as ArrowDataType, Field};
+    use arrow::datatypes::{DataType as ArrowDataType, Field, Schema};
     use arrow::error::Result as ArrowResult;
     use arrow::record_batch::{RecordBatch, RecordBatchReader};
 
-    use crate::arrow::arrow_reader::{ArrowReader, ParquetFileArrowReader};
+    use crate::arrow::arrow_reader::{
+        ArrowReader, ArrowReaderOptions, ParquetFileArrowReader,
+    };
     use crate::arrow::converter::{
         BinaryArrayConverter, Converter, FixedSizeArrayConverter, FromConverter,
         IntervalDayTimeArrayConverter, LargeUtf8ArrayConverter, Utf8ArrayConverter,
     };
     use crate::arrow::schema::add_encoded_arrow_schema_to_metadata;
+    use crate::arrow::ArrowWriter;
     use crate::basic::{ConvertedType, Encoding, Repetition, Type as PhysicalType};
     use crate::column::writer::get_typed_column_writer_mut;
     use crate::data_type::{
@@ -1238,4 +1278,119 @@ mod tests {
         let val = list.value(0);
         assert_eq!(val.len(), 0);
     }
+
+    #[test]
+    fn test_null_schema_inference() {
+        let testdata = arrow::util::test_util::parquet_test_data();
+        let path = format!("{}/null_list.parquet", testdata);
+        let reader =
+            Arc::new(SerializedFileReader::try_from(File::open(&path).unwrap()).unwrap());
+
+        let arrow_field = Field::new(
+            "emptylist",
+            ArrowDataType::List(Box::new(Field::new("item", ArrowDataType::Null, true))),
+            true,
+        );
+
+        let options = ArrowReaderOptions::default().with_skip_arrow_metadata(true);
+        let mut arrow_reader = ParquetFileArrowReader::new_with_options(reader, options);
+        let schema = arrow_reader.get_schema().unwrap();
+        assert_eq!(schema.fields().len(), 1);
+        assert_eq!(schema.field(0), &arrow_field);
+    }
+
+    #[test]
+    fn test_skip_metadata() {
+        let col = Arc::new(TimestampNanosecondArray::from_iter_values(vec![0, 1, 2]));
+        let field = Field::new("col", col.data_type().clone(), true);
+
+        let schema_without_metadata = Arc::new(Schema::new(vec![field.clone()]));
+
+        let metadata = [("key".to_string(), "value".to_string())]
+            .into_iter()
+            .collect();
+
+        let schema_with_metadata =
+            Arc::new(Schema::new(vec![field.with_metadata(Some(metadata))]));
+
+        assert_ne!(schema_with_metadata, schema_without_metadata);
+
+        let batch =
+            RecordBatch::try_new(schema_with_metadata.clone(), vec![col as ArrayRef])
+                .unwrap();
+
+        let file = |version: WriterVersion| {
+            let props = WriterProperties::builder()
+                .set_writer_version(version)
+                .build();
+
+            let file = tempfile().unwrap();
+            let mut writer = ArrowWriter::try_new(
+                file.try_clone().unwrap(),
+                batch.schema(),
+                Some(props),
+            )
+            .unwrap();
+            writer.write(&batch).unwrap();
+            writer.close().unwrap();
+            file
+        };
+
+        let v1_reader = Arc::new(
+            SerializedFileReader::new(file(WriterVersion::PARQUET_1_0)).unwrap(),
+        );
+        let v2_reader = Arc::new(
+            SerializedFileReader::new(file(WriterVersion::PARQUET_2_0)).unwrap(),
+        );
+
+        let mut arrow_reader = ParquetFileArrowReader::new(v1_reader.clone());
+        assert_eq!(
+            &arrow_reader.get_schema().unwrap(),
+            schema_with_metadata.as_ref()
+        );
+
+        let options = ArrowReaderOptions::new().with_skip_arrow_metadata(true);
+        let mut arrow_reader =
+            ParquetFileArrowReader::new_with_options(v1_reader, options);
+        assert_eq!(
+            &arrow_reader.get_schema().unwrap(),
+            schema_without_metadata.as_ref()
+        );
+
+        let mut arrow_reader = ParquetFileArrowReader::new(v2_reader.clone());
+        assert_eq!(
+            &arrow_reader.get_schema().unwrap(),
+            schema_with_metadata.as_ref()
+        );
+
+        let options = ArrowReaderOptions::new().with_skip_arrow_metadata(true);
+        let mut arrow_reader =
+            ParquetFileArrowReader::new_with_options(v2_reader, options);
+        assert_eq!(
+            &arrow_reader.get_schema().unwrap(),
+            schema_without_metadata.as_ref()
+        );
+    }
+
+    #[test]
+    fn test_empty_projection() {
+        let testdata = arrow::util::test_util::parquet_test_data();
+        let path = format!("{}/alltypes_plain.parquet", testdata);
+        let file = File::open(&path).unwrap();
+        let reader = SerializedFileReader::try_from(file).unwrap();
+        let expected_rows = reader.metadata().file_metadata().num_rows() as usize;
+
+        let mut arrow_reader = ParquetFileArrowReader::new(Arc::new(reader));
+        let batch_reader = arrow_reader.get_record_reader_by_columns([], 2).unwrap();
+
+        let mut total_rows = 0;
+        for maybe_batch in batch_reader {
+            let batch = maybe_batch.unwrap();
+            total_rows += batch.num_rows();
+            assert_eq!(batch.num_columns(), 0);
+            assert!(batch.num_rows() <= 2);
+        }
+
+        assert_eq!(total_rows, expected_rows);
+    }
 }
